/*
 *
 *   Copyright (c) 1999 - 2011 my-Channels Ltd
 *   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
 *
 *   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
 *
 */
#include "nAPIConnectionListener.h"
#include "Observer.h"
#include "Observable.h"
#include "Poco/Mutex.h"

#include <string>
#include <map>

namespace com {
namespace pcbsys {
namespace nirvana {
namespace nAdminAPI {
class nRealmACLEntry;
class nConnectionDetails;
class nACL;

namespace apps {

using namespace com::pcbsys::nirvana::nAdminAPI;
using namespace com::pcbsys::foundation::fbase;
using namespace Poco;

class AuthServer;

class LoginMonitor : public nAPIConnectionListener, public Observer {
  private:
    std::map<std::string, std::string> m_hashtable;
    AuthServer *m_pAuth;
    nACL *m_pACL;
    Mutex m_mutex;

  public:
    LoginMonitor(AuthServer *pAuth);
    void update(Observable *pObs, void *pObj);
    virtual void add(nConnectionDetails *pDetails);
    virtual void del(nConnectionDetails *pDetails);

  private:
    void disableUser(const std::string& subject);
    void enableUser(const std::string& subject);
    void removeUser(const std::string& subject);
    bool userSpecificLoginFunction(const std::string& subject, const std::string& protocol);
    void dumpACL(nACL *pAcl);
    void dumpACL(nRealmACLEntry *pEntry);
};

}
}
}
}
}
