#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from Consumer import *

# This is an example of asynchronously subscribing to a channel.  The Consumer base class
# defines several importan functions including the implementation of NirvanaCallback.
# With an asynchronous consumer, the server will push messages to the client as soon
# as they are published onto the channel. The message will be passed to the implementation
# of NirvanaCallback to be processed by the user.
class ChannelSubscriber(Consumer):

    #Override
    def usage(self): 
        print "ChannelSubscriber <rname> <channel name> [start eid] [log level] [count] [selector] "
        print ""
        print "<Required Arguments> "
        print ""
        print "<rname> - the rname of the server to connect to"
        print "<channel name> - Channel name parameter for the channel to subscribe to"
        print ""
        print "[Optional Arguments]"
        print ""
        print "[start eid] - The Event ID to start subscribing from"
        print "[log level] - The level of output from each event, 0 - none, 1 - summary, 2 - EIDs, 3 - properties, 4 - All"
        print "[count] - The number of events to wait before printing out summary information : default 10000"
        print "[selector] - The event filter string to use"

    #Override
    def handleArgs(self, args):
        if (len(args) < 3 ):
            self.usage()
            exit(1)

        self.rname = args[1]
        self.resourcename = args[2]

        if (len(args) > 3 ):
            self.startEid=int(args[3])

        if (len(args) > 4 ):
            self.logLevel = int(args[4])

        if (len(args) > 5 ):
            self.reportCount = int(args[5])

        if (len(args) > 6 ):
            self.selector = args[6]

    #Override
    def doit(self):
        NirvanaModule.subscribeToChannel(self.resourcename,self,self.startEid, self.selector)
        self.inputListener.wait()

    #Override
    def onFinished(self):
        print "Consumed a total of "+str(self.received)


if __name__ == "__main__":
    consumer = ChannelSubscriber(sys.argv)