#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from datetime import *
import threading
import timeit

# This class simply makes it easy to wait and only end a test once a key is pressed
class InputListener(threading.Thread):
    cv = threading.Condition()

    def run(self):
        print "Press any key to quit...\n"
        raw_input()
        self.cv.acquire()
        self.cv.notifyAll()
        self.cv.release()

    def wait(self):
        self.cv.acquire()
        self.cv.wait()
        self.cv.release()

# This is the base class for the asynchronous and synchronous consumer applications. It implements
# NirvanaCallback which has a method "onMessage" which is logs information about any messages received.
class Consumer(NirvanaPython.NirvanaCallback):
    received = 0
    rname = ""
    resourcename = None
    startEid= -1
    logLevel = -1
    reportCount = 10000
    selector = ""
    inputListener = InputListener()

    events = []

    def __init__(self):
        NirvanaPython.NirvanaCallback(self)


    def usage(self):
        print "Consumer Base Class "
        print ""
        print "This is not a sample application, it provides definitions used by sample apps such as "
        print ""
        print "     ChannelSubscriber"
        print "     TXChannelSubcriber"
        print "     QueueSubscriber"
        print "     TXQueueSubscriber"
        print ""

    def displayAttribute(self, name, value):
        if value != "":
            print "  "+str(name)+" : "+str(value)

    def displayProps(self, props, tabs):
        iter = props.getIterator();
        for x in iter:
            if isinstance(x[1], nEventProperties):
                print tabs + str(x[0]) + " properties:"
                self.displayProps(x[1],"  "+tabs)
            else:
                print tabs + str(x[0]) + " : " + str(x[1])

    def onMessage(self,message):
        self.received=self.received+1
        if self.logLevel >= 1:
            print "Received event "+str(self.received)
        if self.logLevel >= 2:
            print "EventID = " +str(message.getEventID())
        if self.logLevel >= 3:
            print "----------------------------------"
            print "Chan name = " +message.getChannelName()
            print "Full chan name = " +message.getFullChannelName()
            print "Timestamp = " +str(message.getTimestamp())
            print "TTL = " +str(message.getTTL())
            print "Has attributes = " +str(message.hasAttributes())
            print "Is end of chan = " +str(message.isEndOfChannel())
            print "Is persistent = " +str(message.isPersistent())
            print "Is protobuf = " +str(message.isProtoBuffEvent())
            print "Is transient = " +str(message.isTransient())
            print "Data = " + str(message.getEventData())
            if message.getProperties() != None:
                print "Properties:"
                self.displayProps(message.getProperties(),"  ")
        if self.logLevel >= 4:
            print "Attributes:"
            self.displayAttribute("allowMerging()",message.getAttributes().allowMerging())
            self.displayAttribute("getApplicationId()",message.getAttributes().getApplicationId())
            self.displayAttribute("getCorrelationId()",message.getAttributes().getCorrelationId())
            self.displayAttribute("getDeadEID()",message.getAttributes().getDeadEID())
            self.displayAttribute("getDeadEventChannel()",message.getAttributes().getDeadEventChannel())
            self.displayAttribute("getDeliveryMode()",message.getAttributes().getDeliveryMode())
            self.displayAttribute("getDestination()",message.getAttributes().getDestination())
            self.displayAttribute("getEventType()",message.getAttributes().getEventType())
            self.displayAttribute("getExpiration()",message.getAttributes().getExpiration())
            self.displayAttribute("getJoinChannel()",message.getAttributes().getJoinChannel())
            self.displayAttribute("getJoinEID()",message.getAttributes().getJoinEID())
            self.displayAttribute("getMessageId()",message.getAttributes().getMessageId())
            self.displayAttribute("getPriority()",message.getAttributes().getPriority())
            self.displayAttribute("getPublisherHost()",message.getAttributes().getPublisherHost())
            self.displayAttribute("getPublisherName()",message.getAttributes().getPublisherName())
            self.displayAttribute("getRedeliveredCount()",message.getAttributes().getRedeliveredCount())
            self.displayAttribute("getReplyToName()",message.getAttributes().getReplyToName())
            self.displayAttribute("getReplyType()",message.getAttributes().getReplyType())
            self.displayAttribute("getSubscriberHost()",message.getAttributes().getSubscriberHost())
            self.displayAttribute("getSubscriberId()",message.getAttributes().getSubscriberId())
            self.displayAttribute("getSubscriberName()",message.getAttributes().getSubscriberName())
            self.displayAttribute("getTimestamp()",datetime.fromtimestamp(message.getAttributes().getTimestamp()//1000))
            self.displayAttribute("getUserId()",message.getAttributes().getUserId())
            self.displayAttribute("isDelta()",message.getAttributes().isDelta())
            self.displayAttribute("isRegistered()",message.getAttributes().isRegistered())
            print "----------------------------------"
        if (self.received % self.reportCount) == 0:
            print "Received "+str(self.received)+" events so far..."

    #Override me
    def handleArgs(self, args):
        print

    #Override me
    def doit(self): 
        print

    #Override me
    def onFinished(self): 
        print

    def establishConnection(self):
        NirvanaModule.connect(self.rname)

    def __init__(self, args):
        try:
            super(Consumer, self).__init__()
            self.handleArgs(args) 

            self.establishConnection()

            self.inputListener.start()
            timer=timeit.Timer(self.doit)

            timetaken = timer.timeit(1)

            print "Finished, time taken = "+str(timetaken)
            self.onFinished()
            NirvanaModule.shutdown()

        except ValueError as e:
            print ""
            print "***** ValueError ***** : %s" % e
            print ""
            self.usage()
            print ""
            print "***** ValueError ***** : %s" % e

if __name__ == "__main__":
    consumer = Consumer(sys.argv)