#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from Publisher import *

# This is an example of how to publish to a data group.  Messages published to a data group
# will be received by any data stream that has been added to that group.
class DataGroupPublisher(Publisher):
    
    #Override
    def usage(self): 
        print ""    
        print "DataGroupPublisher <rname> <group name> [count] [size]" 
        print ""
        print " <Required Arguments>" 
        print "     <rname> - The rname to connect to e.g. nsp://localhost:9000"
        print "     <group name> - Name of the data group to write to"
        print ""
        print " [Optional Arguments]"
        print "     [count] -The number of events to publish (default: 10)"
        print "     [size] - The size (bytes) of the event to publish (default: 100)"

    #Override
    def handleArgs(self, args):
        if (len(args) < 3 ):
            self.usage()
            exit(1)
    
        self.rname = args[1]
        self.resourcename = args[2]

        if (len(args) > 3 ):
            self.count=int(args[3])

        if (len(args) > 4 ):
            self.size = int(args[4])

    #Override
    def onFinished(self):
        print "Successfully published "+str(self.count)

    #Override
    def doit(self):
        print "Starting with publish of "+str(self.count)+" events with a size of "+str(self.size)+" bytes each"

        buffer = bytearray(self.size)
        for x in range(0,self.size):  
            buffer[x] = ((x%90) + 32)
        evt = nConsumeEvent("Tag",str(buffer))

        for x in range(0,self.count):
            NirvanaModule.writeDataGroup(evt,self.resourcename)

if __name__ == "__main__":
    publisher = DataGroupPublisher(sys.argv)