#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from Consumer import *

# This is an example of synchronously consuming events from a Channel.
# Each time a message is received, it is acknowledged.  The server will keep
# track of the last acknowledges message for the supplied named object.
# If you then subscribe again using the same name, you will continue to consume
# from the last acknowledged event.
class TxChannelIterator(Consumer):
    timeout = 1000;
    namedobject = ""

    #Override
    def usage(self): 
        print "TxChannelIterator <rname> <Dest name> <named object> [start eid] [log level] [timeout] [count] [selector] "
        print ""
        print "<Required Arguments> "
        print ""
        print "<rname> - the rname of the server to connect to"
        print "<Dest name> - Channel/Queue name to connect to"
        print "<named object> - Name to use for the named object"
        print ""
        print "[Optional Arguments]"
        print ""
        print "[start eid] - The Event ID to start subscribing from"
        print "[log level] - The level of output from each event, 0 - none, 1 - summary, 2 - EIDs, 3 - properties, 4 - All"
        print "[timeout] - The timeout for the synchronous call (ms) : default 1000"
        print "[count] - The number of events to wait before printing out summary information : default 10000"
        print "[selector] - The event filter string to use"

    #Override
    def handleArgs(self, args):
        if (len(args) < 3 ):
            self.usage()
            exit(1)
    
        self.rname = args[1]
        self.resourcename = args[2]
        self.namedobject = args[3]

        if (len(args) > 4 ):
            self.startEid=int(args[4])

        if (len(args) > 5 ):
            self.logLevel = int(args[5])

        if (len(args) > 6 ):
            self.timeout = int(args[6])

        if (len(args) > 7 ):
            self.reportCount = int(args[7])
    
        if (len(args) > 8 ):
            self.selector = args[8]

    #Override
    def onFinished(self):
        print "Consumed a total of "+str(self.received)

    #Override
    def doit(self):
        iter = NirvanaModule.getChannelIterator(self.resourcename, self.startEid, self.selector, self.namedobject, self.timeout)
        for evt in iter:
            evt.ack()
            self.onMessage(evt)
        # A different use
        #    while True:
        #        evt = iter.next() #can optionally pass a timeout here rather than using the value in the constructor
        #        if evt == None:
        #            print "Timed out, trying again..."
        #        else:
        #            onMessage(evt)
      
        iter.close()

if __name__ == "__main__":
    consumer = TxChannelIterator(sys.argv)