#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from Publisher import *
import time

# This example shows how to transactionally publish to a channel. With a transactional publish,
# the client waits for an acknowledgment from the server that the message has been received.
# If the Universal Messaging server is part of a cluster, it is possible that in the event of a 
# machine failure, the client will disconnect from that machine and reconnect to a different server
# in the cluster.  If this happens, the transaction commit may appear to fail but actually just the
# acknowledgement did not make it to the client. In this situation, the client can query the state
# of the transaction and only recommit if the transaction was unsuccessful.
class TxChannelPublisher(Publisher):
    
    #Override
    def usage(self): 
        print ""    
        print "TxChannelPublisher <rname> <channel name> [count] [size]" 
        print ""
        print " <Required Arguments>" 
        print "     <rname> - The rname to connect to e.g. nsp://localhost:9000"
        print "     <channel name> - Channel name parameter for the channel to publish to"
        print ""
        print " [Optional Arguments]"
        print "     [count] -The number of events to publish (default: 10)"
        print "     [size] - The size (bytes) of the event to publish (default: 100)"

    #Override
    def handleArgs(self, args):
        if (len(args) < 3 ):
            self.usage()
            exit(1)
    
        self.rname = args[1]
        self.resourcename = args[2]

        if (len(args) > 3 ):
            self.count=int(args[3])

        if (len(args) > 4 ):
            self.size = int(args[4])

    #Override
    def onFinished(self):
        print "Successfully published "+str(self.count)

    #Override
    def doit(self):
        print "Starting with publish of "+str(self.count)+" events with a size of "+str(self.size)+" bytes each"

        buffer = bytearray(self.size)
        for x in range(0,self.size):  
            buffer[x] = ((x%90) + 32)

        for x in range(0,self.count):
            evt = nConsumeEvent("evt"+str(x),str(buffer))
            transaction = NirvanaModule.createTransaction(self.resourcename)
            success = False
            for sendtries in range(0, 10) :
                try:
                    transaction.publish(evt)
                    success = True
                    break
                except Exception as e:
                    print("Exception during send : %s "%e)
                    time.sleep(0.1)
            
            if(not success) :
                print("Failed to send event, giving up")
                quit()

            success = False
            for committries in range(0, 10) :
                try:
                    transaction.commit()
                    success = True
                    break
                except Exception as e:
                    print("Exception during commit : %s "%e)
                    isCommitted = transaction.isCommitted()
                    if(isCommitted):
                        success = True #This means the commit was successful but we probably got disconnected before the response was received.
                        break
                    time.sleep(0.1)

            if(not success) :
                print("Failed to commit transaction, giving up")
                quit()

if __name__ == "__main__":
    publisher = TxChannelPublisher(sys.argv)