#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from ChannelSubscriber import *

# This class gives a simple example of asynchronously subscribing to a channel.
# Each time a message is received, it is acknowledged.  The server will keep
# track of the last acknowledges message for the supplied named object.
# If you then subscribe again using the same name, you will continue to consume
# from the last acknowledged event.
class TxChannelSubscriber(Consumer):

    name = ""

    #Override
    def usage(self): 
        print "TxChannelSubscriber <rname> <channel name> <named object> [start eid] [log level] [count] [selector] "
        print ""
        print "<Required Arguments> "
        print ""
        print "<rname> - the rname of the server to connect to"
        print "<channel name> - Channel name parameter for the channel to subscribe to"
        print "<named object> - Name to use for the named object"
        print ""
        print "[Optional Arguments]"
        print ""
        print "[start eid] - The Event ID to start subscribing from"
        print "[log level] - The level of output from each event, 0 - none, 1 - summary, 2 - EIDs, 3 - properties, 4 - All"
        print "[count] - The number of events to wait before printing out summary information : default 10000"
        print "[selector] - The event filter string to use"

    #Override
    def handleArgs(self, args):
        if (len(args) < 4 ):
            self.usage()
            exit(1)

        self.rname = args[1]
        self.resourcename = args[2]
        self.name = args[3]

        if (len(args) > 4 ):
            self.startEid=int(args[4])

        if (len(args) > 5 ):
            self.logLevel = int(args[5])

        if (len(args) > 6 ):
            self.reportCount = int(args[6])

        if (len(args) > 7 ):
            self.selector = args[7]

    #Override
    def doit(self):
        NirvanaModule.subscribeToChannel(self.resourcename,self,self.startEid, self.selector, self.name)
        self.inputListener.wait()

    #Override
    #Here we acknowledge the message to confirm that it was received correctly
    def onMessage(self,message):
        message.ack()
        super(TxChannelSubscriber, self).onMessage(message)

    #Override
    def onFinished(self):
        print "Consumed a total of "+str(self.received)


if __name__ == "__main__":
    consumer = TxChannelSubscriber(sys.argv)