#
#   Copyright (c) 1999 - 2011 my-Channels Ltd
#   Copyright (c) 2012 - 2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.
#
#   Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.
#

from NirvanaModule import *
from Consumer import *

# This class gives a simple example of asynchronously subscribing to a queue
# This is a transactional queue reader so when events are received, they need
# to be acknowledged.  The consumer will only receive a window size of unacknowledged
# events before being required to acknowledge one.  Any unacknowledged events will
# be rolled back onto the queue if the app exits (session closed).
class TxQueueReader(Consumer):

    windowsize = 10

    #Override
    def usage(self): 
        print "TxQueueReader <rname> <queue name> [log level] [count] [selector] [window size] "
        print ""
        print "<Required Arguments> "
        print ""
        print "<rname> - the rname of the server to connect to"
        print "<queue name> - Queue name parameter for the queue to subscribe to"
        print ""
        print "[Optional Arguments]"
        print ""
        print "[log level] - The level of output from each event, 0 - none, 1 - summary, 2 - EIDs, 3 - All"
        print "[count] - The number of events to wait before printing out summary information : default 10000"
        print "[selector] - The event filter string to use"
        print "[window size] - The number of events that can be received before one is acknowledged or rolled back"

    #Override
    def handleArgs(self, args):
        if (len(args) < 3 ):
            self.usage()
            exit(1)

        self.rname = args[1]
        self.resourcename = args[2]

        if (len(args) > 3 ):
            self.logLevel = int(args[3])

        if (len(args) > 4 ):
            self.reportCount = int(args[4])

        if (len(args) > 5 ):
            self.selector = args[5]

        if (len(args) > 6 ):
            self.windowsize = args[6]

    #Override
    def doit(self):
        NirvanaModule.subscribeToQueue(self.resourcename,self, True, self.windowsize)
        self.inputListener.wait()

    #Override
    def onFinished(self):
        print "Consumed a total of "+str(self.received)

    #Override
    #Here we acknowledge the message to confirm that it was received correctly
    def onMessage(self,message):
        message.ack()
        super(TxQueueReader, self).onMessage(message)


if __name__ == "__main__":
    consumer = TxQueueReader(sys.argv)